package main

import (
    "fmt"
    "os"
    "math"
)

func main() {
	var frequency, duty_cycle float64
	var x, ontime_limit int
	var off_value, on_value [88]int
	var clock int = 8000000
	prescaler := [...]int{1, 8, 64, 256, 1024}
	
	fmt.Println("Program for determining compare match values for Timer 1, so piano notes can be replicated. Frequency of each key from 1 to 88 is calculated. Based on Timer 1 settings, corresponding start values are mathed out. Timer 1 is assumed to be 16 bits. Prescaler assumed to be either 1, 8, 64, 256 or 1024.\n")
	
	fmt.Println("Enter AVR Crystal frequency in Hertz, prescaler value (1, 8, 64, 256 or 1024), duty cycle in percent and maximum on-time in micro-seconds. Seperate values with a space.")
	fmt.Scanf("%d %d %f %d", &clock, &x, &duty_cycle, &ontime_limit)
	
	// Get index of prescaler
	for i, _ := range prescaler {
        if (prescaler[i] == x) {
			x = i;
			break;
		}
    }
	
	// create a text file for writing
    fo, err := os.Create("comp_table.txt")
    if err != nil { panic(err) }
    // close fo on exit and check for its returned error
    defer func() {
        if err := fo.Close(); err != nil {
            panic(err)
        }
    }()
	
	stringBuffer := fmt.Sprintf("Generated ON and OFF time values for 16-bit Timer 1. Program created by Eirik Taylor 2013.\n\nAVR Crystal Frequency: %d Hz\nPrescaler: 1:%d\nDuty Cycle: %.2f%%\nOn-time: %d us\n\n\n", clock, prescaler[x], duty_cycle, ontime_limit)
	fo.WriteString(stringBuffer)
	
	duty_cycle = duty_cycle / 100;
	
	// Determine cut-off limit for on-times, to avoid DRSSTC exploding
	resolution := (float64(prescaler[x]) * 1000000000.0) / float64(clock)
	peak_onvalue := (1000*ontime_limit) / int(resolution)
	
	for i := 1; i <= 88; i++ {
		// Find frequency of current note in Hertz
		frequency = 440 * math.Pow(math.Pow(2.0, (1/12.0)), float64(i-49))

		// Find period of frequency measured in timer 1 increments
		frequency = (float64(clock) / (float64(prescaler[x]) * frequency))
		
		// Math out values with chosen duty cycle
		off_value[i-1] = int(frequency)
		
		frequency = (frequency * duty_cycle)
		
		// Set values in array
		on_value[i-1] = int(frequency)
		if (on_value[i-1] > peak_onvalue) {
			on_value[i-1] = peak_onvalue
		}
		off_value[i-1] = off_value[i-1] - on_value[i-1]
	}

	// Print out Off-time values
	stringBuffer = fmt.Sprintf("OFF Values:\n\n")
	for i := 0; i <= 87; i++ {
		if (off_value[i] > 65534) {
			stringBuffer = stringBuffer + fmt.Sprintf("Overflow, ")
		}else {
			stringBuffer = stringBuffer + fmt.Sprintf("%d, ", off_value[i])
		}
	}
	fo.WriteString(stringBuffer)
	
	// Print out On-time values
	stringBuffer = fmt.Sprintf("\n\nON Values:\n\n")
	for i := 0; i <= 87; i++ {
		if (on_value[i] > 65534) {
			stringBuffer = stringBuffer + fmt.Sprintf("Overflow, ")
		}else {
			stringBuffer = stringBuffer + fmt.Sprintf("%d, ", on_value[i])
		}
	}
	fo.WriteString(stringBuffer)

	stringBuffer = fmt.Sprintf("\n\nPut these numbers in the compare_off and compare_on arrays.")
	fo.WriteString(stringBuffer)

	fmt.Printf("\nNo Errors? Check the new list.txt file in this program's directory. :-)\n")
	
}